// Code generated by "stringer -type=protocol -trimprefix=protocol"; DO NOT EDIT.

package proxy

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[protocolAll-0]
	_ = x[protocolNo-1]
	_ = x[protocolHTTP-2]
	_ = x[protocolHTTPS-3]
	_ = x[protocolFTP-4]
	_ = x[protocolSOCKS-5]
	_ = x[protocolAuto-6]
}

const _protocol_name = "AllNoHTTPHTTPSFTPSOCKSAuto"

var _protocol_index = [...]uint8{0, 3, 5, 9, 14, 17, 22, 26}

func (i protocol) String() string {
	if i >= protocol(len(_protocol_index)-1) {
		return "protocol(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _protocol_name[_protocol_index[i]:_protocol_index[i+1]]
}
